from enum import Enum
"""
    0 = CHAR
    1 = SMALLINT
    2 = INTEGER
    3 = FLOAT
    4 = SMALLFLOAT
    5 = DECIMAL
    6 = SERIAL 1
    7 = DATE
    8 = MONEY
    9 = NULL
    10 = DATETIME
    11 = BYTE
    12 = TEXT
    13 = VARCHAR
    14 = INTERVAL
    15 = NCHAR
    16 = NVARCHAR
    17 = INT8
    18 = SERIAL8 1
    19 = SET
    20 = MULTISET
    21 = LIST
    22 = ROW (unnamed)
    23 = COLLECTION
    40 = LVARCHAR fixed-length opaque types 2
    41 = BLOB, BOOLEAN, CLOB variable-length opaque types 2
    43 = LVARCHAR (client-side only)
    45 = BOOLEAN
    52 = BIGINT
    53 = BIGSERIAL 1
    2061 = IDSSECURITYLABEL 2, 3
    4118 = ROW (named)
"""

class InformixTypes(Enum):
    SQL_TYPE_CHAR = 0, 'CharField'
    SQL_TYPE_SMALLINT = 1, 'SmallIntegerField'
    SQL_TYPE_INTEGER = 2, 'IntegerField'
    SQL_TYPE_FLOAT = 3, 'FloatField'
    SQL_TYPE_DOUBLE = 3, 'FloatField'
    SQL_TYPE_REAL = 4, 'FloatField'
    SQL_TYPE_SMFLOAT = 4, 'FloatField'
    SQL_TYPE_DECIMAL = 5, 'DecimalField'
    SQL_TYPE_NUMERIC = 5, 'DecimalField'
    SQL_TYPE_SERIAL = 6, 'AutoField'
    SQL_TYPE_DATE = 7, 'DateField'
    SQL_TYPE_MONEY = 8, 'DecimalField'
    SQL_TYPE_NULL = 9, None
    SQL_TYPE_DATETIME = 10, 'DateTimeField'
    SQL_TYPE_BYTE = 11, 'BinaryField'
    SQL_TYPE_TEXT = 12, 'TextField'
    SQL_TYPE_VARCHAR = 13, 'CharField'
    SQL_TYPE_INTERVAL = 14, 'DurationField'
    SQL_TYPE_NCHAR = 15, 'CharField'
    SQL_TYPE_NVARCHAR = 16, 'CharField'
    SQL_TYPE_INT8 = 17, 'IntegerField'
    SQL_TYPE_SERIAL8 = 18, 'AutoField'
    SQL_TYPE_SET = 19, None
    SQL_TYPE_MASK = 31, None
    SQL_TYPE_UDTVAR = 40, 'CharField'
    SQL_TYPE_UDTFIXED = 41, None
    SQL_TYPE_LVARCHAR = 43, 'CharField'
    SQL_TYPE_BOOLEAN = 45, 'BoolField'
    SQL_TYPE_BIGINT = 52, 'BigIntegerField'
    SQL_TYPE_BIG_SERIAL = 53, 'AutoField'

    def __init__(self, num, field = None):
        self.num = num
        self.field = field


    @classmethod
    def field_map(cls):
        return {e.num: e.field for e in cls}



from django.db import models


class Donut(models.Model):
    name = models.CharField(max_length=100)
    is_frosted = models.BooleanField(default=False)
    cost = models.DecimalField(decimal_places=2, max_digits=10, default=0)



from django.db import models


class TrimCharField(models.CharField):
    description = "CharField that ignores trailing spaces in data"

    def from_db_value(self, value, expression, connection, context):
        if value:
            return value.rstrip()
        return value



from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.utils import ProgrammingError, DatabaseError


class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):

    sql_create_unique = "ALTER TABLE %(table)s ADD CONSTRAINT UNIQUE (%(columns)s) CONSTRAINT %(name)s "
    sql_create_fk = (
        "ALTER TABLE %(table)s ADD CONSTRAINT FOREIGN KEY (%(column)s) "
        "REFERENCES %(to_table)s (%(to_column)s) ON DELETE CASCADE CONSTRAINT  %(name)s"
    )
    sql_create_column = "ALTER TABLE %(table)s ADD %(column)s %(definition)s"
    sql_alter_column_null = "MODIFY %(column)s %(type)s NULL"
    sql_alter_column_not_null = "MODIFY %(column)s %(type)s NOT NULL"
    sql_alter_column_type = "MODIFY %(column)s %(type)s"
    sql_alter_column_default = "MODIFY %(column)s DEFAULT "
    sql_alter_column_no_default = "MODIFY %(column)s DROP DEFAULT"
    sql_delete_column = "ALTER TABLE %(table)s DROP %(column)s"

    def execute(self, sql, params=[]):
        """
        Informix adds an index to foreign keys automatically

        This silences the error when Django tries to do the same thing independently
        """
        try:
            super(DatabaseSchemaEditor, self).execute(sql, params)
        except (ProgrammingError, DatabaseError) as e:
            if "CREATE INDEX" not in str(sql) and 'Index already exists' not in str(e):
                # ugh, that feels dirty
                raise e

    def skip_default(self, field):
        """
        It's not easy to handle defaults easily for informix because of its weird syntax
        An example for this is to add default for some datetime field:

        ALTER TABLE django_admin_log MODIFY ( action_time datetime year to fraction(5)
        NOT NULL DEFAULT DATETIME(2016-05-25 00:26:23.00000) year to fraction(5));
        """
        return True



from django.db.backends.base.creation import BaseDatabaseCreation
from django.db.utils import Error


class DatabaseCreation(BaseDatabaseCreation):

    def sql_table_creation_suffix(self):
        test_settings = self.connection.settings_dict['TEST']
        assert test_settings['COLLATION'] is None, (
            "Informix does not support collation setting at database creation time."
        )
        if test_settings['CHARSET']:
            return "WITH ENCODING '%s'" % test_settings['CHARSET']
        return 'WITH BUFFERED LOG'

    def create_test_db(self, verbosity=1, autoclobber=False, serialize=True, keepdb=False):
        """
        Forcing autoclobber because destroying test DB doesn't work

        The keepdb option still works
        """
        if self.connection.settings_dict.get('TEST', {}).get('CREATE_DB', True):
            super().create_test_db(verbosity, True, serialize, keepdb)

    def _destroy_test_db(self, test_database_name, verbosity):
        if self.connection.settings_dict.get('TEST', {}).get('CREATE_DB', True):
            try:
                super()._destroy_test_db(test_database_name, verbosity)
            except Error as e:
                print(f"Unable to destroy test database: {e.args[1]}")



from setuptools import setup, find_packages
from codecs import open
from os import path

from django_informixdb import __version__
here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='django_informixdb',
    version=__version__,
    description='A database driver for Django to connect to an Informix db via ODBC',
    long_description=long_description,
    url='https://github.com/reecetech/django_informixdb',
    author='Reecetech',
    author_email='opensource@reecetech.com.au',
    license='APLv2',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'Topic :: Scientific/Engineering',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='django informix',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=['django>=1.9.6', 'pyodbc==4.0.21'],
    extras_require={
        'dev': ['check-manifest'],
        'test': ['coverage'],
    },
)



from decimal import Decimal

from django.test import TestCase

from .models import Donut


class DataTypesTestCase(TestCase):
    def test_boolean_false(self):
        d = Donut(name='Apple Fritter')
        self.assertFalse(d.is_frosted)
        d.save()
        d2 = Donut.objects.get(name='Apple Fritter')
        self.assertFalse(d2.is_frosted)

    def test_boolean_true(self):
        d = Donut(name='Apple Fritter', is_frosted=True)
        self.assertTrue(d.is_frosted)
        d.save()
        d2 = Donut.objects.get(name='Apple Fritter')
        self.assertTrue(d2.is_frosted)

    def test_decimal(self):
        d = Donut(name='Apple Fritter', cost=1)
        self.assertEqual(d.cost, 1)
        d.save()
        d2 = Donut.objects.get(name='Apple Fritter')
        self.assertEqual(d2.cost, Decimal(1))

    def test_decimal_precision(self):
        d = Donut(name='Apple Fritter', cost=1.23)
        self.assertEqual(d.cost, 1.23)
        d.save()
        d2 = Donut.objects.get(name='Apple Fritter')
        self.assertEqual(d2.cost, Decimal('1.23'))
